from os.path import exists
from os import makedirs
from sys import version
if version[0] == '2':
    from urllib2 import urlopen
    import sys
    reload(sys)
    sys.setdefaultencoding("utf8")
else:
    from urllib.request import urlopen


def font_check(font_path, font_list):
    if not exists(font_path):
        makedirs(font_path)
    target = []
    for font in font_list:
        if not exists(font_path + '/{}'.format(font)):
            target.append(font)
    return target


def font_downloader(base_url, font_name, font_path):
    with open(font_path + '/{}'.format(font_name), 'w') as f_handle:
        handle = urlopen(base_url + font_name)
        f_handle.write(handle.read())


def font_handle(font_path, font_list, base_url):
    target = font_check(font_path, font_list)
    if target:
        for font in target:
            print(font + ' is downloading...')
            font_downloader(base_url, font, font_path)
            print(font + ' downloaded~~')
        return None
    print('all font is downloaded~~')





# coding=utf8
from paramSeeker import ParamSeeker
from printer import Printer
from extras import font_handle, font_check
from os import path

seeker = ParamSeeker()
printer = Printer()

font_list = ['DejaVuSansMono-Bold.ttf',
             'handstd_h.otf',
             'fengyun.ttf',
             'huakangbold.otf',
             'letter.ttf',
             'shuyan.ttf']

font_path = printer.font_location

default = {"text": "HellFlame",
           "Type": "",
           "mode": "text",
           "color": "31",
           "Filter": 14,
           "font": 0,
           "file": '',
           "force": False}


@seeker.seek(param='--init', short='-i', is_mark=True, extra={'desc': 'download fonts and init program'})
def init_program(wanted):
    check = font_handle(font_path, font_list, base_url='http://7xqh1q.dl1.z0.glb.clouddn.com/')
    if check:
        print(check)
    exit(0)


@seeker.seek(param='--text', short='-t', extra={'desc': '设置将要处理的文本内容，默认为HellFlame'})
def text_content(wanted):
    default['text'] = wanted
    return ''


@seeker.seek(param='--type', short='-T', extra={'desc': '强制指定文本类型，默认为系统自动判别，若指定为英文，则 -T en，其他类型时输入其他'})
def font_type(wanted):
    default['Type'] = wanted
    default['force'] = True
    return ''


@seeker.seek(param='--mode', short='-m', extra={'desc': '设置输出模式, text表示文本输出；color表示按某种颜色输出，颜色值由-c指定；r_color表示使用随机颜色填充'})
def mode_set(wanted):
    default['mode'] = wanted
    return ''


@seeker.seek(param='--color', short='-c', extra={'desc': "若模式选择为color，则指定将要输出的颜色,默认为{},输入值范围为30 ～ 50，"
                                                         "不排除某些值没有对应颜色的可能,虽然你也可以输入其他值试一试".format(default["color"])})
def color_set(wanted):
    default['color'] = wanted
    return ''


@seeker.seek(param='--filter', short='-f', extra={'desc': "设置打印填充方式，可选择1-56的数值，输出内容的精细程度会随数值的增大而变细致，"
                                                          "但这还跟终端窗口的大小以及屏幕分辨率有关，过高的填充值会造成画面破碎"})
def filter_set(wanted):
    default['Filter'] = wanted
    return ''


@seeker.seek(param='--font', short='-F', extra={'desc': "设置输出字体，结果可能与文本类型有关"})
def font_set(wanted):
    default['font'] = wanted
    return ''


@seeker.seek(extra={'default': '.'})
def pic_handle(wanted):
    if wanted == '.':
        if font_check(font_path, font_list):
            print('请运行 -i or --init 初始化字体库\n或者-h or --help 帮助')
            return ''
        else:
            return ''
    else:
        if path.exists(wanted):
            default['file'] = wanted
            return ''
        return '米有找到→_→ {} ←_←这个文件的说喵'.format(wanted)


seeker.set_desc("terminal text or image printer")
seeker.set_usage_desc("terminalprint -[tFmcf] [param,]")
seeker.set_usage_desc("terminalprint picture.jpg")
seeker.set_usage_desc("terminalprint -t linux -m r_color")
seeker.set_usage_desc("terminalprint -t 测试 -m color -c 5")
seeker.set_usage_desc("terminalprint -t 女王大人 -F 1 -m r_color")


def runner():
    seeker.run(no_output=True)
    if default['file']:
        printer.set_img(default['file'])
        pic_str = printer.make_char_img(filter_type=default['Filter'])

    else:
        if default['Type']:
            auto = False
        else:
            auto = True
        printer.text_drawer(text=default['text'], lang=default['Type'], font_choice=default['font'], auto=auto)
        printer.set_img(printer.tmp_pic)
        pic_str = printer.make_char_img(default['Filter'])

    if default['mode'] == 'color':
        print(printer.dye_all(pic_str, default['color']))
    elif default['mode'] == 'r_color':
        print(printer.dye_rand(pic_str))
    else:
        print(pic_str)


if __name__ == '__main__':
    runner()




#!/bin/bash
# coding=utf8

from PIL import Image, ImageFont, ImageDraw
from random import randrange
from os import popen, environ


class Printer:
    def __init__(self, w=0, h=0):
        self.filter_type = '.~-_+*^?/%$!@( #&`\\)|1234567890abcdefghijklmnopqrstuvwxyz'
        self.font_location = environ['HOME'] + '/.terminal_fonts/'
        self.img = None
        self.tmp_pic = '/tmp/printer_{}.png'.format(environ['USER'])
        if w and h:
            self.console_w = w
            self.console_h = h
        else:
            console = popen("stty size").read().split()
            self.console_w = int(console[1])
            self.console_h = int(console[0])

    def make_char_img(self, filter_type=14):
        if not self.img:
            return False
        pix = self.img.load()
        pic_str = ''
        width, height = self.img.size
        for h in xrange(height):
            for w in xrange(width):
                pic_str += self.filter_type[int(pix[w, h]) * int(filter_type) / 255]
            pic_str += '\n'
        return pic_str

    def set_img(self, file_path):
        img = Image.open(file_path)
        img = img.resize((self.console_w, self.console_h))
        self.img = img.convert('L')

    def text_drawer(self, text, lang, font_choice=0, auto=False):
        text_len = len(text)
        tmp_pic = self.tmp_pic
        im = Image.new("1", (self.console_w, self.console_h), 'white')
        draw = ImageDraw.Draw(im)
        mark = False

        if auto and self.simple_lang(text) == 'en':
            fontsize = 20
            font = ((self.font_location + "DejaVuSansMono-Bold.ttf", int(text_len * fontsize * 0.63), int(fontsize * 1.15)),
                    (self.font_location + "handstd_h.otf", int(text_len * fontsize * 0.55), int(fontsize)),
                    (self.font_location + "shuyan.ttf", int(text_len * fontsize * 0.5), int(fontsize * 1.2)),
                    (self.font_location + "fengyun.ttf", int(text_len * fontsize * 0.68), int(fontsize * 1.3))
                    )
        else:
            mark = True
            fontsize = 20
            font = ((self.font_location + "letter.ttf", int(text_len * fontsize * 0.32), int(fontsize * 1.1)),
                    (self.font_location + "shuyan.ttf", int(text_len * fontsize * 0.35), int(fontsize * 1.2)),
                    (self.font_location + "huakangbold.otf", int(text_len * fontsize * 0.34), int(fontsize * 1.4)),
                    (self.font_location + "fengyun.ttf", int(text_len * fontsize * 0.34), int(fontsize * 1.4))
                    )
        Max = 3
        font_choice = int(font_choice)
        if 0 <= font_choice <= Max:
            font, width, height = font[font_choice]
        elif font_choice > Max:
            font, width, height = font[Max]
        else:
            font, width, height = font[0]

        font = ImageFont.truetype(font, fontsize)
        text_size = draw.textsize(text, font=font)
        if mark:
            im = im.resize((width, height))
        else:
            im = im.resize(text_size)

        draw = ImageDraw.Draw(im)
        draw.text((0, 0), unicode(text), font=font)
        im.save(tmp_pic)

    @staticmethod
    def getlang(text):
        lang_list = []
        for i in text:
            if unichr(ord(i)) == i.decode('utf8', errors='ignore'):
                lang_list.append({
                    'lang': 'en',
                    'content': i
                })
            else:
                lang_list.append({
                    'lang': 'other',
                    'content': i
                })
        char_count = len(lang_list)
        if char_count == 1:
            return lang_list
        result = []
        mark = 0
        for i in range(char_count)[1:]:
            if lang_list[i - 1]['lang'] == lang_list[i]['lang']:
                if i == 1:
                    result.append(lang_list[i - 1])
                    result[mark]['content'] += lang_list[i]['content']
                else:
                    result[mark]['content'] += lang_list[i]['content']
            else:
                if i == 1:
                    result.append(lang_list[i - 1])
                result.append(lang_list[i])
                mark += 1
        return result

    @staticmethod
    def simple_lang(text):
        for i in text:
            if unichr(ord(i)) != i.decode('utf8', errors='ignore'):
                return 'other'
        return 'en'

    @staticmethod
    def dye(string, color):
        return color + string + "\033[1;m"

    def dye_all(self, string, color):
        temp = ""
        str_list = string.split("\n")
        for i in str_list:
            temp += self.dye(i, color)
            if str_list.index(i) != 1:
                temp += "\n"
        return temp

    @staticmethod
    def dye_rand(string):
        temp = ""
        for i in string:
            if i != "\n":
                temp += "\033[{};{}m{}\033[1;m".format(randrange(1, 4), randrange(30, 40), i)
        return temp

    @staticmethod
    def get_color(tail):
        return "\033[3;{}m".format(tail)






# coding=utf8
__author__ = 'hellflame'

from setuptools import setup, find_packages


setup(
    name='TerminalPrinter',
    version='1.1.3',
    keywords=('text printer', 'picture printer', 'picture in terminal', 'print picture in terminal'),
    description="文字,字符,图片终端打印, print something in terminal",
    license='Apache License',
    author='hellflame',
    author_email='hellflamedly@gmail.com',
    url='https://github.com/hellflame/terminal_printer',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2.7',
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Topic :: Text Processing :: General'
    ],
    install_requires=[
        'Pillow',
        'paramSeeker'
    ],
    platforms="linux, Mac Os",
    entry_points={
        'console_scripts': [
            'terminalprint=terminal_printer.runner:runner'
        ]
    }
)


